// Copyright (c) 2022 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#pragma once

//------------------------------------------------------------------------------

#include "Acceptor.h"

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

/**
 * Socket acceptor template for a certain kind of client sockets.
 */
template <class Socket>
class SocketAcceptor : public Acceptor
{
protected:
    /**
     * Construct the acceptor.
     */
    SocketAcceptor(ServerSocket* socket);

public:
    /**
     * Get the socket accepted last. If no socket was accepted,
     * return 0. The socket's waiter will be the given one
     * (which can be 0).
     */
    Socket* getSocket(Waiter* waiter,
                      size_t readingCapacity = Socket::DEFAULT_CAPACITY,
                      size_t writingCapacity = Socket::DEFAULT_CAPACITY);
    /**
     * Get the socket accepted last. If no socket was accepted,
     * return 0. The socket's waiter will be the same as that of
     * the associated server socket.
     */
    Socket* getSocket(size_t readingCapacity = Socket::DEFAULT_CAPACITY,
                      size_t writingCapacity = Socket::DEFAULT_CAPACITY);

    /**
     * Get the last accepted socket as a buffered stream with the given waiter,
     * if any.
     */
    BufferedStream* getAcceptedBufferedStream(Waiter* waiter = nullptr) override;
};

//------------------------------------------------------------------------------
// Template definitions
//------------------------------------------------------------------------------

template <class Socket>
inline SocketAcceptor<Socket>::SocketAcceptor(ServerSocket* socket) :
    Acceptor(socket)
{
}

//------------------------------------------------------------------------------

template <class Socket>
inline Socket*
SocketAcceptor<Socket>::getSocket(Waiter* waiter,
                                  size_t readingCapacity,
                                  size_t writingCapacity)
{
    if (acceptedFD<0) return 0;

    Socket* socket = new Socket(acceptedFD, waiter,
                                readingCapacity, writingCapacity);
    connectionProcessed();
    return socket;
}

//------------------------------------------------------------------------------

template <class Socket>
inline Socket*
SocketAcceptor<Socket>::getSocket(size_t readingCapacity,
                                  size_t writingCapacity)
{
    return getSocket(socket.getWaiter(), readingCapacity, writingCapacity);
}

//------------------------------------------------------------------------------

template <class Socket>
BufferedStream* SocketAcceptor<Socket>::
getAcceptedBufferedStream(Waiter* waiter)
{
    return (waiter==nullptr) ? getSocket() : getSocket(waiter);
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
